/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState, useRef, useEffect } from 'react';
import { useAuthorizationCache, useAuth, useUserRole } from '@graphpolaris/shared/lib/data-access';
import DatabaseSelector from './DatabaseManagement/dbConnectionSelector';
import { DropdownContainer, DropdownTrigger, DropdownItemContainer, DropdownItem } from '@graphpolaris/shared/lib/components/dropdowns';
import GpLogo from './gp-logo';
import { Popover, PopoverContent, PopoverTrigger } from '@graphpolaris/shared/lib/components/layout/Popover';
import { useDispatch } from 'react-redux';
import { setRole, Role } from '@graphpolaris/shared/lib/data-access/store/userSlice';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import {
  Dialog,
  DialogContent,
  DialogHeading,
  DialogTrigger,
  DialogDescription,
  DialogClose,
} from '@graphpolaris/shared/lib/components/layout/Dialog';
import { UserManagementContent } from '@graphpolaris/shared/lib/components/userManagementContent/UserManagementContent';

export const Navbar = () => {
  const dropdownRef = useRef<HTMLDivElement>(null);
  const auth = useAuth();
  const authCache = useAuthorizationCache();
  const [menuOpen, setMenuOpen] = useState(false);
  const dispatch = useDispatch();
  const [toggleValue, setToggleValue] = useState(false);
  const userRole = useUserRole();
  const [selectedValue, setSelectedValue] = useState(userRole);
  const buildInfo = import.meta.env.GRAPHPOLARIS_VERSION;

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (dropdownRef.current && !dropdownRef.current.contains(event.target as Node)) {
        setMenuOpen(false);
      }
    };
    if (menuOpen) document.addEventListener('mousedown', handleClickOutside);
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, [menuOpen]);

  const handleItemClick = (role: Role) => {
    setSelectedValue(role);
    dispatch(setRole(role));
  };

  const handleToggleChange = (newValue: boolean) => {
    console.log(newValue);
    setToggleValue(newValue);
    if (newValue) {
      handleItemClick('viewer');
    } else {
      handleItemClick('creator');
    }
  };

  return (
    <nav className="w-full px-4 h-12 flex flex-row items-center gap-2 md:gap-3 lg:gap-4">
      <a href="https://graphpolaris.com/" target="_blank" className="shrink-0 text-dark">
        <GpLogo className="h-7" />
      </a>
      <DatabaseSelector />
      <div className="ml-auto">
        <div className="w-fit" ref={dropdownRef}>
          <Popover>
            <PopoverTrigger>
              <div
                className="relative inline-flex items-center justify-center w-8 h-8 overflow-hidden bg-secondary-500 rounded-full hover:bg-secondary-600 transition-colors duration-150 ease-in-out cursor-pointer"
                onClick={() => setMenuOpen(!menuOpen)}
              >
                <span className="font-medium text-light">{authCache.username?.slice(0, 2).toUpperCase()}</span>
              </div>
            </PopoverTrigger>

            <PopoverContent className="w-56 z-30 bg-white rounded-sm border-[1px] outline-none">
              <div className="p-2 text-sm border-b">
                <h2 className="font-bold">user: {authCache.username}</h2>
                <h3 className="text-xs break-words">session: {authCache.sessionID}</h3>
                <h3 className="text-xs break-words">license: Creator</h3>
              </div>

              {authCache.authorized ? (
                <>
                  <DropdownItem
                    value="Share"
                    onClick={() => {
                      auth.newShareRoom();
                    }}
                  />
                  <DropdownItem value="Settings" onClick={() => {}} />
                  <DropdownItem value="Log out" onClick={() => {}} />
                </>
              ) : (
                <>
                  <DropdownItem value="Login" onClick={() => {}} />
                </>
              )}

              {authCache?.roomID && (
                <div className="p-2 border-b">
                  <h3 className="text-xs break-words">Share ID: {authCache.roomID}</h3>
                </div>
              )}
              <div className="p-2 border-t">
                <h3 className="text-xs">Version: {buildInfo}</h3>
              </div>
              <Dialog>
                <DialogTrigger className="ml-2 text-sm">Manage Viewers Permission</DialogTrigger>
                <DialogContent>
                  <UserManagementContent
                    sessionId={authCache.sessionID ?? ''}
                    onConfirm={() => console.log('Confirmed')}
                    onCancel={() => console.log('Cancelled')}
                  />
                </DialogContent>
              </Dialog>
              <div className="ml-2">
                <Input
                  label="Check Viewer Role"
                  type="toggle"
                  value={toggleValue}
                  classText="text-sm"
                  onChange={handleToggleChange}
                  tooltip="Toggle this switch to enable or disable the Viewer role"
                />
              </div>
            </PopoverContent>
          </Popover>
        </div>
      </div>
    </nav>
  );
};
